/* Any copyright is dedicated to the Public Domain.
  http://creativecommons.org/publicdomain/zero/1.0/ */
/* eslint-disable max-len */

"use strict";

/*
 * THIS FILE IS AUTOGENERATED. DO NOT MODIFY BY HAND. RUN TESTS IN FIXTURES/ TO UPDATE.
 */

const { ConsoleMessage } =
  require("devtools/client/webconsole/types");

const stubPreparedMessages = new Map();
const stubPackets = new Map();
stubPreparedMessages.set(`new Date(0)`, new ConsoleMessage({
  "id": "1",
  "innerWindowID": null,
  "allowRepeating": true,
  "source": "javascript",
  "timeStamp": 1479159921364,
  "type": "result",
  "helperType": null,
  "level": "log",
  "category": null,
  "parameters": [
    {
      "type": "object",
      "actor": "server1.conn0.child1/obj30",
      "class": "Date",
      "extensible": true,
      "frozen": false,
      "sealed": false,
      "ownPropertyLength": 0,
      "preview": {
        "timestamp": 0
      }
    }
  ],
  "repeatId": "{\"frame\":null,\"groupId\":null,\"indent\":0,\"level\":\"log\",\"parameters\":[{\"type\":\"object\",\"actor\":\"server1.conn0.child1/obj30\",\"class\":\"Date\",\"extensible\":true,\"frozen\":false,\"sealed\":false,\"ownPropertyLength\":0,\"preview\":{\"timestamp\":0}}],\"source\":\"javascript\",\"type\":\"result\",\"userProvidedStyles\":null}",
  "frame": null,
  "groupId": null,
  "cssSelectors": "",
  "userProvidedStyles": null,
  "notes": null,
  "indent": 0,
  "prefix": "",
  "chromeContext": false
}));

stubPreparedMessages.set(`asdf()`, new ConsoleMessage({
  "id": "1",
  "innerWindowID": null,
  "allowRepeating": true,
  "source": "javascript",
  "timeStamp": 1479159921377,
  "type": "result",
  "helperType": null,
  "level": "error",
  "category": null,
  "messageText": "ReferenceError: asdf is not defined",
  "parameters": [
    {
      "type": "undefined"
    }
  ],
  "repeatId": "{\"frame\":{\"source\":\"debugger eval code\",\"line\":1,\"column\":1},\"groupId\":null,\"indent\":0,\"level\":\"error\",\"messageText\":\"ReferenceError: asdf is not defined\",\"parameters\":[{\"type\":\"undefined\"}],\"source\":\"javascript\",\"type\":\"result\",\"userProvidedStyles\":null,\"stacktrace\":[{\"filename\":\"debugger eval code\",\"sourceId\":\"server1.conn0.child1/source23\",\"lineNumber\":1,\"columnNumber\":1,\"functionName\":null}]}",
  "stacktrace": [
    {
      "filename": "debugger eval code",
      "sourceId": "server1.conn0.child1/source23",
      "lineNumber": 1,
      "columnNumber": 1,
      "functionName": null
    }
  ],
  "frame": {
    "source": "debugger eval code",
    "line": 1,
    "column": 1
  },
  "groupId": null,
  "errorMessageName": "JSMSG_NOT_DEFINED",
  "exceptionDocURL": "https://developer.mozilla.org/docs/Web/JavaScript/Reference/Errors/Not_defined?utm_source=mozilla&utm_medium=firefox-console-errors&utm_campaign=default",
  "cssSelectors": "",
  "userProvidedStyles": null,
  "notes": null,
  "indent": 0,
  "prefix": "",
  "chromeContext": false
}));

stubPreparedMessages.set(`1 + @`, new ConsoleMessage({
  "id": "1",
  "innerWindowID": null,
  "allowRepeating": true,
  "source": "javascript",
  "timeStamp": 1479159921399,
  "type": "result",
  "helperType": null,
  "level": "error",
  "category": null,
  "messageText": "SyntaxError: illegal character",
  "parameters": [
    {
      "type": "undefined"
    }
  ],
  "repeatId": "{\"frame\":{\"source\":\"debugger eval code\",\"line\":1,\"column\":4},\"groupId\":null,\"indent\":0,\"level\":\"error\",\"messageText\":\"SyntaxError: illegal character\",\"parameters\":[{\"type\":\"undefined\"}],\"source\":\"javascript\",\"type\":\"result\",\"userProvidedStyles\":null,\"stacktrace\":null}",
  "stacktrace": null,
  "frame": {
    "source": "debugger eval code",
    "line": 1,
    "column": 4
  },
  "groupId": null,
  "errorMessageName": "JSMSG_ILLEGAL_CHARACTER",
  "exceptionDocURL": "https://developer.mozilla.org/docs/Web/JavaScript/Reference/Errors/Illegal_character?utm_source=mozilla&utm_medium=firefox-console-errors&utm_campaign=default",
  "cssSelectors": "",
  "userProvidedStyles": null,
  "notes": null,
  "indent": 0,
  "prefix": "",
  "chromeContext": false
}));

stubPreparedMessages.set(`inspect({a: 1})`, new ConsoleMessage({
  "id": "1",
  "innerWindowID": null,
  "allowRepeating": true,
  "source": "javascript",
  "timeStamp": 1499776070751,
  "type": "result",
  "helperType": "inspectObject",
  "level": "log",
  "category": null,
  "parameters": [
    {
      "type": "object",
      "actor": "server1.conn0.child1/obj35",
      "class": "Object",
      "extensible": true,
      "frozen": false,
      "sealed": false,
      "ownPropertyLength": 1,
      "preview": {
        "kind": "Object",
        "ownProperties": {
          "a": {
            "configurable": true,
            "enumerable": true,
            "writable": true,
            "value": 1
          }
        },
        "ownSymbols": [],
        "ownPropertiesLength": 1,
        "ownSymbolsLength": 0,
        "safeGetterValues": {}
      }
    }
  ],
  "repeatId": "{\"frame\":null,\"groupId\":null,\"indent\":0,\"level\":\"log\",\"parameters\":[{\"type\":\"object\",\"actor\":\"server1.conn0.child1/obj35\",\"class\":\"Object\",\"extensible\":true,\"frozen\":false,\"sealed\":false,\"ownPropertyLength\":1,\"preview\":{\"kind\":\"Object\",\"ownProperties\":{\"a\":{\"configurable\":true,\"enumerable\":true,\"writable\":true,\"value\":1}},\"ownSymbols\":[],\"ownPropertiesLength\":1,\"ownSymbolsLength\":0,\"safeGetterValues\":{}}}],\"source\":\"javascript\",\"type\":\"result\",\"userProvidedStyles\":null}",
  "frame": null,
  "groupId": null,
  "cssSelectors": "",
  "userProvidedStyles": null,
  "notes": null,
  "indent": 0,
  "prefix": "",
  "chromeContext": false
}));

stubPreparedMessages.set(`cd(document)`, new ConsoleMessage({
  "id": "1",
  "innerWindowID": null,
  "allowRepeating": true,
  "source": "javascript",
  "timeStamp": 1510650094657,
  "type": "result",
  "helperType": "error",
  "level": "error",
  "category": null,
  "messageText": "Cannot cd() to the given window. Invalid argument.",
  "parameters": [
    {
      "type": "undefined"
    }
  ],
  "repeatId": "{\"frame\":null,\"groupId\":null,\"indent\":0,\"level\":\"error\",\"messageText\":\"Cannot cd() to the given window. Invalid argument.\",\"parameters\":[{\"type\":\"undefined\"}],\"source\":\"javascript\",\"type\":\"result\",\"userProvidedStyles\":null}",
  "frame": null,
  "groupId": null,
  "cssSelectors": "",
  "userProvidedStyles": null,
  "notes": null,
  "indent": 0,
  "prefix": "",
  "chromeContext": false
}));

stubPreparedMessages.set(`undefined`, new ConsoleMessage({
  "id": "1",
  "innerWindowID": null,
  "allowRepeating": true,
  "source": "javascript",
  "timeStamp": 1518606917356,
  "type": "result",
  "helperType": null,
  "level": "log",
  "category": null,
  "parameters": [
    {
      "type": "undefined"
    }
  ],
  "repeatId": "{\"frame\":null,\"groupId\":null,\"indent\":0,\"level\":\"log\",\"parameters\":[{\"type\":\"undefined\"}],\"source\":\"javascript\",\"type\":\"result\",\"userProvidedStyles\":null}",
  "frame": null,
  "groupId": null,
  "cssSelectors": "",
  "userProvidedStyles": null,
  "notes": null,
  "indent": 0,
  "prefix": "",
  "chromeContext": false
}));

stubPreparedMessages.set(`longString message Error`, new ConsoleMessage({
  "id": "1",
  "innerWindowID": null,
  "allowRepeating": true,
  "source": "javascript",
  "timeStamp": 1493108241073,
  "type": "result",
  "helperType": null,
  "level": "error",
  "category": null,
  "messageText": {
    "type": "longString",
    "initial": "Error: Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Lon",
    "length": 110007,
    "actor": "server1.conn0.child1/longString37"
  },
  "parameters": [
    {
      "type": "undefined"
    }
  ],
  "repeatId": "{\"frame\":{\"source\":\"debugger eval code\",\"sourceId\":\"server1.conn0.child1/source30\",\"line\":1,\"column\":7},\"groupId\":null,\"indent\":0,\"level\":\"error\",\"messageText\":{\"type\":\"longString\",\"initial\":\"Error: Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Lon\",\"length\":110007,\"actor\":\"server1.conn0.child1/longString37\"},\"parameters\":[{\"type\":\"undefined\"}],\"source\":\"javascript\",\"type\":\"result\",\"userProvidedStyles\":null,\"stacktrace\":[{\"filename\":\"debugger eval code\",\"sourceId\":\"server1.conn0.child1/source30\",\"lineNumber\":1,\"columnNumber\":7,\"functionName\":null}]}",
  "stacktrace": [
    {
      "filename": "debugger eval code",
      "sourceId": "server1.conn0.child1/source30",
      "lineNumber": 1,
      "columnNumber": 7,
      "functionName": null
    }
  ],
  "frame": {
    "source": "debugger eval code",
    "sourceId": "server1.conn0.child1/source30",
    "line": 1,
    "column": 7
  },
  "groupId": null,
  "cssSelectors": "",
  "userProvidedStyles": null,
  "notes": null,
  "indent": 0,
  "prefix": "",
  "chromeContext": false
}));

stubPreparedMessages.set(`eval throw ""`, new ConsoleMessage({
  "id": "1",
  "innerWindowID": null,
  "allowRepeating": true,
  "source": "javascript",
  "timeStamp": 1517990289517,
  "type": "result",
  "helperType": null,
  "level": "error",
  "category": null,
  "messageText": "Error",
  "parameters": [
    {
      "type": "undefined"
    }
  ],
  "repeatId": "{\"frame\":{\"source\":\"debugger eval code\",\"sourceId\":\"server1.conn0.child1/source23\",\"line\":1,\"column\":1},\"groupId\":null,\"indent\":0,\"level\":\"error\",\"messageText\":\"Error\",\"parameters\":[{\"type\":\"undefined\"}],\"source\":\"javascript\",\"type\":\"result\",\"userProvidedStyles\":null,\"stacktrace\":[{\"filename\":\"debugger eval code\",\"sourceId\":\"server1.conn0.child1/source23\",\"lineNumber\":1,\"columnNumber\":1,\"functionName\":null}]}",
  "stacktrace": [
    {
      "filename": "debugger eval code",
      "sourceId": "server1.conn0.child1/source23",
      "lineNumber": 1,
      "columnNumber": 1,
      "functionName": null
    }
  ],
  "frame": {
    "source": "debugger eval code",
    "sourceId": "server1.conn0.child1/source23",
    "line": 1,
    "column": 1
  },
  "groupId": null,
  "cssSelectors": "",
  "userProvidedStyles": null,
  "notes": null,
  "indent": 0,
  "prefix": "",
  "chromeContext": false
}));

stubPreparedMessages.set(`eval throw "tomato"`, new ConsoleMessage({
  "id": "1",
  "innerWindowID": null,
  "allowRepeating": true,
  "source": "javascript",
  "timeStamp": 1517990289520,
  "type": "result",
  "helperType": null,
  "level": "error",
  "category": null,
  "messageText": "Error: tomato",
  "parameters": [
    {
      "type": "undefined"
    }
  ],
  "repeatId": "{\"frame\":{\"source\":\"debugger eval code\",\"sourceId\":\"server1.conn0.child1/source23\",\"line\":1,\"column\":1},\"groupId\":null,\"indent\":0,\"level\":\"error\",\"messageText\":\"Error: tomato\",\"parameters\":[{\"type\":\"undefined\"}],\"source\":\"javascript\",\"type\":\"result\",\"userProvidedStyles\":null,\"stacktrace\":[{\"filename\":\"debugger eval code\",\"sourceId\":\"server1.conn0.child1/source23\",\"lineNumber\":1,\"columnNumber\":1,\"functionName\":null}]}",
  "stacktrace": [
    {
      "filename": "debugger eval code",
      "sourceId": "server1.conn0.child1/source23",
      "lineNumber": 1,
      "columnNumber": 1,
      "functionName": null
    }
  ],
  "frame": {
    "source": "debugger eval code",
    "sourceId": "server1.conn0.child1/source23",
    "line": 1,
    "column": 1
  },
  "groupId": null,
  "cssSelectors": "",
  "userProvidedStyles": null,
  "notes": null,
  "indent": 0,
  "prefix": "",
  "chromeContext": false
}));

stubPackets.set(`new Date(0)`, {
  "from": "server1.conn0.child1/consoleActor2",
  "input": "new Date(0)",
  "result": {
    "type": "object",
    "actor": "server1.conn0.child1/obj30",
    "class": "Date",
    "extensible": true,
    "frozen": false,
    "sealed": false,
    "ownPropertyLength": 0,
    "preview": {
      "timestamp": 0
    }
  },
  "timestamp": 1479159921364,
  "exception": null,
  "frame": null,
  "helperResult": null,
  "notes": null
});

stubPackets.set(`asdf()`, {
  "from": "server1.conn0.child1/consoleActor2",
  "input": "asdf()",
  "result": {
    "type": "undefined"
  },
  "timestamp": 1479159921377,
  "exception": {
    "type": "object",
    "actor": "server1.conn0.child1/obj32",
    "class": "Error",
    "extensible": true,
    "frozen": false,
    "sealed": false,
    "ownPropertyLength": 4,
    "preview": {
      "kind": "Error",
      "name": "ReferenceError",
      "message": "asdf is not defined",
      "stack": "@debugger eval code:1:1\n",
      "fileName": "debugger eval code",
      "lineNumber": 1,
      "columnNumber": 1
    }
  },
  "exceptionMessage": "ReferenceError: asdf is not defined",
  "exceptionDocURL": "https://developer.mozilla.org/docs/Web/JavaScript/Reference/Errors/Not_defined?utm_source=mozilla&utm_medium=firefox-console-errors&utm_campaign=default",
  "exceptionStack": [
    {
      "filename": "debugger eval code",
      "sourceId": "server1.conn0.child1/source23",
      "lineNumber": 1,
      "columnNumber": 1,
      "functionName": null
    }
  ],
  "errorMessageName": "JSMSG_NOT_DEFINED",
  "frame": {
    "source": "debugger eval code",
    "line": 1,
    "column": 1
  },
  "helperResult": null,
  "notes": null
});

stubPackets.set(`1 + @`, {
  "from": "server1.conn0.child1/consoleActor2",
  "input": "1 + @",
  "result": {
    "type": "undefined"
  },
  "timestamp": 1479159921399,
  "exception": {
    "type": "object",
    "actor": "server1.conn0.child1/obj33",
    "class": "Error",
    "extensible": true,
    "frozen": false,
    "sealed": false,
    "ownPropertyLength": 4,
    "preview": {
      "kind": "Error",
      "name": "SyntaxError",
      "message": "illegal character",
      "stack": "",
      "fileName": "debugger eval code",
      "lineNumber": 1,
      "columnNumber": 4
    }
  },
  "exceptionMessage": "SyntaxError: illegal character",
  "exceptionDocURL": "https://developer.mozilla.org/docs/Web/JavaScript/Reference/Errors/Illegal_character?utm_source=mozilla&utm_medium=firefox-console-errors&utm_campaign=default",
  "exceptionStack": null,
  "errorMessageName": "JSMSG_ILLEGAL_CHARACTER",
  "frame": {
    "source": "debugger eval code",
    "line": 1,
    "column": 4
  },
  "helperResult": null,
  "notes": null
});

stubPackets.set(`inspect({a: 1})`, {
  "from": "server1.conn0.child1/consoleActor2",
  "input": "inspect({a: 1})",
  "result": {
    "type": "undefined"
  },
  "timestamp": 1499776070751,
  "exception": null,
  "frame": null,
  "helperResult": {
    "type": "inspectObject",
    "input": "inspect({a: 1})",
    "object": {
      "type": "object",
      "actor": "server1.conn0.child1/obj35",
      "class": "Object",
      "extensible": true,
      "frozen": false,
      "sealed": false,
      "ownPropertyLength": 1,
      "preview": {
        "kind": "Object",
        "ownProperties": {
          "a": {
            "configurable": true,
            "enumerable": true,
            "writable": true,
            "value": 1
          }
        },
        "ownSymbols": [],
        "ownPropertiesLength": 1,
        "ownSymbolsLength": 0,
        "safeGetterValues": {}
      }
    }
  },
  "notes": null
});

stubPackets.set(`cd(document)`, {
  "from": "server1.conn0.child1/consoleActor2",
  "input": "cd(document)",
  "result": {
    "type": "undefined"
  },
  "timestamp": 1510650094657,
  "exception": null,
  "frame": null,
  "helperResult": {
    "type": "error",
    "message": "cdFunctionInvalidArgument"
  },
  "notes": null
});

stubPackets.set(`undefined`, {
  "from": "server1.conn0.child1/consoleActor2",
  "input": "undefined",
  "result": {
    "type": "undefined"
  },
  "timestamp": 1518606917356,
  "exception": null,
  "frame": null,
  "helperResult": null,
  "notes": null
});

stubPackets.set(`longString message Error`, {
  "from": "server1.conn0.child1/consoleActor2",
  "input": "throw new Error(\"Long error \".repeat(10000))",
  "result": {
    "type": "undefined"
  },
  "timestamp": 1493108241073,
  "exception": {
    "type": "object",
    "actor": "server1.conn0.child1/obj35",
    "class": "Error",
    "extensible": true,
    "frozen": false,
    "sealed": false,
    "ownPropertyLength": 4,
    "preview": {
      "kind": "Error",
      "name": "Error",
      "message": {
        "type": "longString",
        "initial": "Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error",
        "length": 110000,
        "actor": "server1.conn0.child1/longString36"
      },
      "stack": "@debugger eval code:1:7\n",
      "fileName": "debugger eval code",
      "lineNumber": 1,
      "columnNumber": 7
    }
  },
  "exceptionMessage": {
    "type": "longString",
    "initial": "Error: Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Lon",
    "length": 110007,
    "actor": "server1.conn0.child1/longString37"
  },
  "exceptionStack": [
    {
      "filename": "debugger eval code",
      "sourceId": "server1.conn0.child1/source30",
      "lineNumber": 1,
      "columnNumber": 7,
      "functionName": null
    }
  ],
  "frame": {
    "source": "debugger eval code",
    "sourceId": "server1.conn0.child1/source30",
    "line": 1,
    "column": 7
  },
  "helperResult": null,
  "notes": null
});

stubPackets.set(`eval throw ""`, {
  "from": "server1.conn0.child1/consoleActor2",
  "input": "throw \"\"",
  "result": {
    "type": "undefined"
  },
  "timestamp": 1517990289517,
  "exception": "",
  "exceptionMessage": "",
  "exceptionStack": [
    {
      "filename": "debugger eval code",
      "sourceId": "server1.conn0.child1/source23",
      "lineNumber": 1,
      "columnNumber": 1,
      "functionName": null
    }
  ],
  "frame": {
    "source": "debugger eval code",
    "sourceId": "server1.conn0.child1/source23",
    "line": 1,
    "column": 1
  },
  "helperResult": null,
  "notes": null
});

stubPackets.set(`eval throw "tomato"`, {
  "from": "server1.conn0.child1/consoleActor2",
  "input": "throw \"tomato\"",
  "result": {
    "type": "undefined"
  },
  "timestamp": 1517990289520,
  "exception": "tomato",
  "exceptionMessage": "tomato",
  "exceptionStack": [
    {
      "filename": "debugger eval code",
      "sourceId": "server1.conn0.child1/source23",
      "lineNumber": 1,
      "columnNumber": 1,
      "functionName": null
    }
  ],
  "frame": {
    "source": "debugger eval code",
    "sourceId": "server1.conn0.child1/source23",
    "line": 1,
    "column": 1
  },
  "helperResult": null,
  "notes": null
});

module.exports = {
  stubPreparedMessages,
  stubPackets,
};
