
/*
** Copyright (c) 2012 The Khronos Group Inc.
**
** Permission is hereby granted, free of charge, to any person obtaining a
** copy of this software and/or associated documentation files (the
** "Materials"), to deal in the Materials without restriction, including
** without limitation the rights to use, copy, modify, merge, publish,
** distribute, sublicense, and/or sell copies of the Materials, and to
** permit persons to whom the Materials are furnished to do so, subject to
** the following conditions:
**
** The above copyright notice and this permission notice shall be included
** in all copies or substantial portions of the Materials.
**
** THE MATERIALS ARE PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
** EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
** MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
** CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
** TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
** MATERIALS OR THE USE OR OTHER DEALINGS IN THE MATERIALS.
*/


#ifdef GL_ES
precision mediump float;
#endif
//
// mat3arrayindirect0_frag.frag: Fragment shader solid color
// The vec3 values are determined at runtime.
//
//

uniform mat3 testmat3[2];
varying vec4  color;

void main(void)
{
     vec3 result = vec3(0.0, 0.0, 0.0);

  /*
	// No indirect indexing in fragment shaders
     for(int j = 0; j < 3; j++)
     {
	result += testmat3[0][j] + testmat3[1][j];
     }
	*/
	result += testmat3[0][0] + testmat3[1][0];
	result += testmat3[0][1] + testmat3[1][1];
	result += testmat3[0][2] + testmat3[1][2];
     gl_FragColor = vec4(result/2.0, 0.5);
}
